﻿using MathNet.Numerics.LinearAlgebra;
using System;
using System.Linq;

using np = PythonUtils;
using vector = MathNet.Numerics.LinearAlgebra.Vector<double>;
using matrix = MathNet.Numerics.LinearAlgebra.Matrix<double>;
using System.Collections.Generic;

class Program
{
    static void Main(string[] args)
    {
        var X = np.arange(-1.0, 1.0, 0.1).ToArray();
        var Y = np.arange(-1.0, 1.0, 0.1).ToArray();

        List<double> x1 = new List<double>();
        List<double> x2 = new List<double>();
        List<double> y1 = new List<double>();
        List<double> y2 = new List<double>();

        matrix w_im = GetMatrix(new[] { 1.0, 2.0 }, new[] { 2.0, 3.0 });
        matrix w_mo = GetMatrix(new[] { -1.0, 1.0 }, new[] { 1.0, -1.0 });

        vector b_im = GetVector(0.3, -0.3);
        vector b_mo = GetVector(0.4, 0.1);

        Func<vector, matrix, vector, vector> middle_layer = (x, w, b) =>
        {
            vector u = x * w + b;
            return 1 / (1 + np.exp(-u));
        };

        Func<vector, matrix, vector, vector> output_layer = (x, w, b) =>
        {
            vector u = x * w + b;
            return np.exp(u) / np.sum(u);
        };

        for (int i = 0; i < X.Length; i++)
        {
            for (int j = 0; j < Y.Length; j++)
            {
                var inp = GetVector(X[i], Y[j]);
                var mid = middle_layer(inp, w_im, b_im);
                var outp = output_layer(mid, w_mo, b_mo);

                if (outp[0] > outp[1])
                {
                    x1.Add(X[i]);
                    y1.Add(Y[j]);
                }
                else
                {
                    x2.Add(X[i]);
                    y2.Add(Y[j]);
                }
            }
        }

        OutputImage("layer3_neuron_classification.png");

        void OutputImage(string fileName)
        {
            Gridmap grid = new Gridmap(471, 471);
            grid.Show(x1.ToArray(), y1.ToArray(), x2.ToArray(), y2.ToArray(), fileName);
        }
    }

    private static Matrix<double> GetMatrix(params double[][] values)
    {
        return CreateMatrix.DenseOfRows(values.Length, values[0].Length, values);
    }

    private static Vector<double> GetVector(params double[] values)
    {
        return CreateVector.DenseOfArray(values);
    }
}
